#========================================================================================
# package      DE_EPAGES::Shippo::t::GrantShippoAuthorization_si
# base         DE_EPAGES::Shop::API::TestImportXML
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration Test for GrantShippoAuthorization
#========================================================================================
package DE_EPAGES::Shippo::t::GrantShippoAuthorization_si;
use base DE_EPAGES::Shop::API::TestImportXML;

use Readonly;
use URI;
use URI::QueryParam;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Shippo::API::Connector;
use DE_EPAGES::Shippo::API::User qw ( CreateShippoUser GetShippoAuthToken );
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::Shippo::UI::Shop;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use strict;

use constant SHIPPING_METHOD_NAME_IN_SHOP => 'Change this name';
use constant SHIPPO_ID => '3453564646454';
use constant SHIPPING_TYPE_FIX_PRICE => 'ShippingMethodFixPrice';

# test plan
sub test {
    my $self = shift;

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = GetPKeyLanguageByCode('en');

    $self->test_ShippoCredentials();

    done_testing();
}

sub test_ShippoCredentials {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Password = 'password';
    my $ShopPath = $self->shopPath();
    my $Shop = LoadObjectByPath($ShopPath);

    my $Connector = $self->_createConnector($Shop);
    my $User = $self->_createShippoUser($Shop, $Password);
    my $AuthToken = $self->_getShippoAuthToken($Shop,$Connector,$User, $Password);
    my $ShippingFulfillmentType = $self->_getShippingFulfillmentType($Shop);

    like($User->pathString, '/ShippoConnector/' ,"$SubTestName - Check that a user called ShippoConnector is created");
    like($Connector->GetShippoSoapURL($Shop), '/soap$/' ,"$SubTestName - Check Shipo Soap URL sent"); ## no critic qw(RequireInterpolationOfMetachars)
    like($AuthToken, '/^[A-Z0-9]+$/i', "$SubTestName - Check that Authtoken contains data"); ## no critic qw(RequireInterpolationOfMetachars)
    is($ShippingFulfillmentType->get('AuthorizationURL'), 'https://goshippo.com/epages/auth/connect/',
       "$SubTestName - We are getting the URL of Shippo where the credentials have to be sent");
}

sub shopAlias {
    return 'TestShopShippo';
}

sub importXMLPath {
    return 'ShopXMLs/ShopWithShippo.xml';
}

#========================================================================================
sub _createShippoUser {
    my $self = shift;
    my ($Shop,$Password) = @_;
    my $User = CreateShippoUser($Shop, $Password);
    return $User;
}

#========================================================================================
sub _getShippoAuthToken {
    my $self = shift;
    my ($Shop,$Connector,$User,$Password) = @_;
    my $AuthToken = GetShippoAuthToken($User->pathString, $Password, $Connector->GetShippoSoapURL($Shop));
    return $AuthToken;
}

#========================================================================================
sub _getShippingFulfillmentType {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippoProviderMethod = $self->_getShippoShippingProviderMethod($Shop);
    my $ShippingFulfillmentType = $ShippoProviderMethod->get('ShippingFulfillmentType');

    return $ShippingFulfillmentType;
}

#========================================================================================
sub _createConnector {
    my $self = shift;
    my ($Shop) = @_;

    my $ShippoShippingProviderMethod = $self->_getShippoShippingProviderMethod($Shop);
    return DE_EPAGES::Shippo::API::Connector->new($ShippoShippingProviderMethod);
}

#========================================================================================
sub _getShippoShippingProviderMethod {
    my $self = shift;
    my ($Shop) = @_;

    my $ProviderMethodsFolder = $Shop->folder('ShippingProviderMethods');
    my $ShippoProviderMethod = $ProviderMethodsFolder->existsChild('Shippo') ? $ProviderMethodsFolder->child('Shippo') : undef;

    return $ShippoProviderMethod;
}

__PACKAGE__->runScript();

1;
