#========================================================================================
# package      DE_EPAGES::Shippo::t::unit::CredentialsShippo
# base         DE_EPAGES::Core::API::Test
# state        public
# description  unit test for DE_EPAGES::Shippo::API::CredentialsShippo
#========================================================================================
package DE_EPAGES::Shippo::t::unit::CredentialsShippo;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Shippo::API::Connector;
use DE_EPAGES::Shippo::API::CredentialsShippo;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::Test::Mock::Mockify;

use strict;

use constant SHIPPO_ID => '3453564646454';

#----------------------------------------------------------------------------------------
sub test {
    my $self = shift;
    $self->_test_createShippoCredentials();
    done_testing();
}

sub _test_createShippoCredentials {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $isShippingProviderMethod   = '1';
    my $MockedShippingProviderMethodShippo = $self->_createShippingProviderMethodShippo();
    my $MockedShippingMethodShippo = $self->_createShippingMethodShippo($isShippingProviderMethod,$MockedShippingProviderMethodShippo);
    my $MockedForm                 = $self->_createForm();
    my $MockedShop                 = $self->_createShop();
    my $MockedCredentialsShippo    = $self->_createCredentialsShippo($MockedShippingMethodShippo);

    my $ShippoIdBefore = $MockedShippingProviderMethodShippo->get('ShippoID');

    is( $ShippoIdBefore, '', "$SubTestName - tests if SHIPPO_ID has not been set yet" );

    my $areCredentialsCreated = $MockedCredentialsShippo->createShippoCredentials( $MockedForm, $MockedShop );
    is ($areCredentialsCreated, '1', "$SubTestName - Credentials have been successfully created" );
    my $ShippoId = $MockedShippingProviderMethodShippo->get('ShippoID');
    is( $ShippoId, SHIPPO_ID, "$SubTestName - tests if SHIPPO_ID has been set" );
}

#----------------------------------------------------------------------------------------
sub _createCredentialsShippo {
    my $self = shift;
    my ($MockedShippingMethodShippo) = @_;

    my $CredentialsShippo       = DE_EPAGES::Shippo::API::CredentialsShippo->new($MockedShippingMethodShippo);
    my $MockedCredentialsShippo = Test::MockObject::Extends->new($CredentialsShippo);

    $MockedCredentialsShippo->mock(
        '_GenerateNewPassword',
        sub {
            shift;
            return '1234567890';
        }
    );

    $MockedCredentialsShippo->mock(
        '_createConnector',
        sub {
            shift;
            my $MockedShippingProviderMethodShippo = $self->_createShippingProviderMethodShippo();
            return $self->_createConnector($MockedShippingProviderMethodShippo);
        }
    );

    $MockedCredentialsShippo->mock(
        '_CreateShippoUser',
        sub {
            shift;

            my $hAtributes = { 'PathString' => '/Shops/DemoShop/Users/ShippoConnector', };

            return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Permission::API::Object::User', $hAtributes );
        }
    );

    $MockedCredentialsShippo->mock(
        '_GetShippoAuthToken',
        sub {
            shift;
            return 'ddyhfrtfwrt3gwrt3gwrt3whh589789';
        }
    );

    $MockedCredentialsShippo->mock(
        '_LoadRootObject',
        sub {
            shift;
            my $hAttributes = { 'ShippoProviderKey' => 'gs45345sdgfs45345hsdq34578dfgd', };
            my $MockedSystem =
              DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::Site', $hAttributes );
            return $MockedSystem;
        }
    );

}

#----------------------------------------------------------------------------------------
sub _createShippingMethodShippo {
    my $self = shift;
    my ($isShippingProviderMethod,$MockedShippingProviderMethodShippo) = @_;

    my $MockedShippingMethodShippo =
      DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shippo::API::ShippingMethodShippo');

    $MockedShippingMethodShippo->mock(
        'getShippingProviderMethodShippo',
        sub {
            shift;
            return $MockedShippingProviderMethodShippo;
        }
    );

    return $MockedShippingMethodShippo;
}

#----------------------------------------------------------------------------------------
sub _createShippingProviderMethodShippo {
    my $self = shift;

    my $hAtributes = { 'ShippingFulfillmentType' => $self->_createShippingFulfillmentType(), 'ShippoID' => '' };

    my $MockedShippingProviderMethodShippo =
      DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shippo::API::Object::ShippingProviderMethodShippo',
        $hAtributes );

    return $MockedShippingProviderMethodShippo;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentType {
    my $self = shift;

    my $MockedShippingFulfillmentTypeShippo =
      DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shippo::API::Object::ShippingFulfillmentTypeShippo');

    return $MockedShippingFulfillmentTypeShippo;
}

#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;

    my $MockedForm = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Presentation::API::Form');

    $MockedForm->mock(
        'value',
        sub {
            my $self     = shift;
            my ($hValue) = @_;
            my $shippoID = SHIPPO_ID;
            return $shippoID;

        }
    );

    return $MockedForm;
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop');

    return $MockedShop;
}

#----------------------------------------------------------------------------------------
sub _createConnector {
    my $self = shift;
    my ($MockedShippingMethodShippo) = @_;

    my $Connector       = DE_EPAGES::Shippo::API::Connector->new($MockedShippingMethodShippo);
    my $MockedConnector = Test::MockObject::Extends->new($Connector);

    $MockedConnector->mock(
        'SendShippoAuthCredentials',
        sub {
            my $self = shift;
            return;

        }
    );

    return $MockedConnector;
}

__PACKAGE__->runScript();

1;
